using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.IO;
using System.Runtime.InteropServices;	// Needed for IOleCommandTarget
using System.Text;
using System.Xml.XPath;
using System.Windows.Forms;
using Microsoft.Office.InfoPath;
using Microsoft.Office.Interop.InfoPath;
using HostClassLibrary;

namespace InsuranceForms
{
    public partial class Form1 : Form, IInitEventHandler, INotifyHostEventHandler, ISubmitToHostEventHandler
	{
		const int AutoInsurance = 1;
		const int HomeOwnersInsurance = 2;
		const int LifeInsurance = 3;
		const int NoSelection = 0;
        const string HostDisplayName = "IP Insurance";

        enum FindDirection
        {
            up = 1,
            down = 2,
            entireDocument = 3,
        }

		private string _currentDir = System.IO.Directory.GetCurrentDirectory();
		private string _currentFileName = null;
		private int _currentlySelectedForm = NoSelection;
        private DateTime _lastIdleTime;
        private OLECMD[] _commands;

        public Form1()
		{
			InitializeComponent();
            EnsureCommandsArray();
        }

		// IOleCommandTarget implementation
		#region IOleCommandTarget_Implementation
		/// <summary>
		/// OLECMD structure - see MSDN for more information
		/// </summary>
		[StructLayout(LayoutKind.Sequential)]
		struct OLECMD
		{
			public uint cmdID;
			public uint cmdf;
		}

		/// <summary>
		/// OLECMDF Enum - see MSDN for more information
		/// </summary>
		private enum OleCmdf
		{
			Supported	= 1,	// The command is supported
			Enabled		= 2,	// The command is available and enabled
			Latched		= 4,	// The command is a toggle (on-off) and is current on
			Ninched		= 8		// Reserved for future use
		}

        // Note that the GUID created using guidgen tool that comes 
        // with Visual Studio 2005
        [
        ComImport,
        Guid("B722BCCB-4E68-101B-A2BC-00AA00404770"),
        InterfaceType(ComInterfaceType.InterfaceIsIUnknown)
        ]
        interface IOleCommandTarget
        {
            void QueryStatus(ref Guid pguidCmdGroup, uint cCmds, [MarshalAs(UnmanagedType.LPArray), In, Out] OLECMD[] prgCmds, IntPtr pCmdText);
            void Exec(ref Guid pguidCmdGroup, uint nCmdID, uint nCmdExecOpt, ref object pvaIn, ref object pvaOut);
        }
		#endregion

		/// <summary>
		/// ExecuteCommand - Executes the given IOleCommandTarget command
		/// </summary>
		/// <param name="cmd">Command ID</param>
		/// <param name="parameters">Parameters to pass with the command.</param>
		/// <param name="output">Output from the command.</param>
        private bool ExecuteCommand(FormControlCommandIds.CommandIds cmd, object parameters, ref object output)
		{
			// Don't do anything if the command is not available
            if (QueryStatusCommand(cmd))
            {
                // Get the command target object from the form control
                IOleCommandTarget cmdTarget = formControl1.GetOcx() as IOleCommandTarget;

                try
                {
                    cmdTarget.Exec(ref FormControlCommandIds.CommandGroup, Convert.ToUInt16(cmd), 0, ref parameters, ref output);
                }
                catch (COMException)
                {
                    return false;
                }
                catch (Exception e)
                {
                    MessageBox.Show(e.ToString(), HostDisplayName);
                    return false;
                }

                return true;
            }
            else
            {
                return false;
            }
		}

		/// <summary>
		/// ExecuteCommand - Executes the given IOleCommandTarget command
		/// </summary>
		/// <param name="cmd">Command ID</param>
        private bool ExecuteCommand(FormControlCommandIds.CommandIds cmd)
		{
			object output = null;
			return ExecuteCommand(cmd, null, ref output);
		}

		/// <summary>
		/// QueryStatusCommand - Queries for the current status of the given command ID.
		/// </summary>
		/// <param name="cmd">Command ID</param>
		/// <returns>bool - true if command is enabled, false otherwise</returns>
        public bool QueryStatusCommand(FormControlCommandIds.CommandIds cmd)
		{
			// Get the command target object from the form control
			if (formControl1 != null)
			{
				IOleCommandTarget commandTarget = formControl1.GetOcx() as IOleCommandTarget;

				// Create an array for the OLECMD
				OLECMD[] oleCommands = new OLECMD[1];

				oleCommands[0].cmdID = Convert.ToUInt16(cmd);
				oleCommands[0].cmdf = 0;	// flags to pass with the command (taken from the OLECMDF enumeration)

				try
				{
					commandTarget.QueryStatus(ref FormControlCommandIds.CommandGroup, 1, oleCommands, IntPtr.Zero);
					return (((oleCommands[0].cmdf & (uint)OleCmdf.Supported) != 0) && ((oleCommands[0].cmdf & (uint)OleCmdf.Enabled) != 0));
				}
				catch (Exception e)
				{
                    MessageBox.Show(e.ToString(), HostDisplayName);
				}
			}

			return false;
		}

        /// <summary>
        /// SetToolbarState - Queries for the current status of the
        /// basic toolbar commands and sets their states accordingly.
        /// </summary>
        public void EnsureCommandsArray()
        {
            FormControlCommandIds.CommandIds[] formControlCommands =
            {
                FormControlCommandIds.CommandIds.Cut,
                FormControlCommandIds.CommandIds.Copy,
                FormControlCommandIds.CommandIds.Paste,
                FormControlCommandIds.CommandIds.Undo,
                FormControlCommandIds.CommandIds.Redo,
            };

            _commands = new OLECMD[formControlCommands.Length];

            for (int i = 0; i < formControlCommands.Length; i++)
            {
                _commands[i].cmdID = Convert.ToUInt16(formControlCommands[i]);
                _commands[i].cmdf = 0;
            }
        }

        /// <summary>
        /// SetToolbarState - Queries for the current status of the
        /// basic toolbar commands and sets their states accordingly.
        /// </summary>
        public void SetToolBarButtonStates()
        {
            if (formControl1 != null)
            {
                try
                {
                    ToolStripButton[] buttons = 
                    {
                        cutButton,
                        copyButton,
                        pasteButton,
                        undoButton,
                        redoButton
                    };

                    IOleCommandTarget commandTarget = formControl1.GetOcx() as IOleCommandTarget;

                    commandTarget.QueryStatus(
                        ref FormControlCommandIds.CommandGroup, 
                        Convert.ToUInt16(_commands.Length), 
                        _commands, 
                        IntPtr.Zero);

                    for (int i = 0; i < _commands.Length; i++)
                    {
                        bool enabled = (((_commands[i].cmdf & (uint)OleCmdf.Supported) != 0) &&
                                       ((_commands[i].cmdf & (uint)OleCmdf.Enabled) != 0));

                        buttons[i].Enabled = enabled;
                    }
                    
                    // Finally, set the state of the Save button. 
                    // The open button is always enabled.
                    saveButton.Enabled = (formControl1.XmlForm != null) 
                                      && formControl1.XmlForm.Dirty;
                }
                catch (Exception e)
                {
                    MessageBox.Show(e.ToString(), HostDisplayName);
                }
            }
        }

        private void comboBox1_SelectedIndexChanged(object sender, EventArgs e)
		{
			ComboBox comboBox = (ComboBox)sender;
            string formTemplate = null;

			if (comboBox.SelectedIndex == _currentlySelectedForm)
			{
				return;
			}

			switch (comboBox.SelectedIndex)
			{
				case AutoInsurance:
					formTemplate = "AutoInsurance.xsn";
					break;

				case HomeOwnersInsurance:
                    formTemplate = "HomeOwnersInsurance.xsn";
					break;

				case LifeInsurance:
                    formTemplate = "LifeInsurance.xsn";
					break;
			}

			bool shouldClose = SaveIfDirty();
			if (shouldClose)
			{
                // Close the form before opening a new one.
                formControl1.Close();
				_currentlySelectedForm = NoSelection; // None option (0)

                if (formTemplate != null)
				{
                    // NewFromFormTemplate will throw an exception if the 
                    // form cannot be opened.
                    try
                    {
                        formControl1.NewFromFormTemplate(_currentDir + "\\Forms\\" + formTemplate);
                    }
                    catch (Exception ex)
                    {
                        MessageBox.Show(ex.Message, HostDisplayName);
                    }

                    _currentlySelectedForm = comboBox.SelectedIndex;
				}
				else
				{
                    // Otherwise, hide the form control.
                    formControl1.Visible = false;
					EnableDisableToolbarButtons(false);
				}
			}
			else
			{
				// Reset the previous selection
				comboBox.SelectedIndex = _currentlySelectedForm;
			}
		}

		private void Form1_Load(object sender, EventArgs e)
		{
            comboBox1.SelectedIndex = NoSelection;

            formControl1.Host = new HostUtilities();
            formControl1.HostName = HostDisplayName;
            formControl1.Visible = false;

            formControl1.SetInitEventHandler(this);
            formControl1.SetNotifyHostEventHandler(this);
            formControl1.SetSubmitToHostEventHandler(this);

            formControl1.InternalStartup += new FormControl.EventHandler<EventArgs>(formControl1_InternalStartup);

            // Disable the buttons until a form is opened
            EnableDisableToolbarButtons(false);

            // Set up idle handle in order to enable/disable toolbar buttons
            _lastIdleTime = DateTime.Now;
            System.Windows.Forms.Application.Idle += new EventHandler(Application_Idle);
        }

        void formControl1_InternalStartup(object sender, EventArgs e)
        {
            try
            {
                formControl1.EventManager.XmlEvents["/my:myFields/my:InsurerName"].Changed 
                    += new XmlChangedEventHandler(InsurerNameChanged);
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, HostDisplayName);
            }
        }

        void InsurerNameChanged(object sender, XmlEventArgs e)
        {
            MessageBox.Show("The insurer name has changed", HostDisplayName);
        }

        void Application_Idle(object sender, EventArgs e)
        {
            // Don't update the UI on every idle call. Wait at least 
            // one second in between each update.
            if (DateTime.Now > _lastIdleTime.AddSeconds(1))
            {
                this.SuspendLayout();
                SetToolBarButtonStates();
                this.ResumeLayout();

                _lastIdleTime = DateTime.Now;
            }
        }

		private void Form1_Closing(object sender, System.ComponentModel.CancelEventArgs e)
		{
			bool shouldClose = SaveIfDirty();
			e.Cancel = !shouldClose;
		}

		private void EnableDisableToolbarButtons(bool enable)
		{
            findTextBox.Enabled = enable;
		}

		#region HostEventHandlers
        public void InitEventHandler(object sender, XmlForm xmlForm, out XdReadOnlyViewMode viewsReadOnlyMode)
		{
            formControl1.Visible = true;
			EnableDisableToolbarButtons(true);

            viewsReadOnlyMode = XdReadOnlyViewMode.xdDefault;
        }

        public void NotifyHostEventHandler(object sender, string notification)
        {
            if (notification == "FormReady")
	        {
                MessageBox.Show("The form is ready", HostDisplayName);
            }
        }

        public int SubmitToHostEventHandler(object sender, string adapterName, out string errorMessage)
		{
			bool formIsValid = true;
            errorMessage = null;

            if (formControl1.XmlForm != null)
            {
                XPathNavigator navigator = formControl1.XmlForm.MainDataSource.CreateNavigator();

                XPathNavigator insurerNameNode = navigator.SelectSingleNode(
                    "my:myFields/my:InsurerName", formControl1.XmlForm.NamespaceManager);

                if (insurerNameNode != null && insurerNameNode.Value == "")
                {
                    formIsValid = false;
                    errorMessage = "The insurer name cannot be empty";
                }
            }

            if (formIsValid)
			{
				EmailSubmitConnection emailSubmit = formControl1.XmlForm.DataConnections["Email Submit"] as EmailSubmitConnection;
				if (emailSubmit != null)
				{
					emailSubmit.Execute();
				}
			}

            return formIsValid ? 1 : 0;
		}
		#endregion

		#region Clipboard Handlers
		private void cutButton_Click(object sender, EventArgs e)
		{
            ExecuteCommand(FormControlCommandIds.CommandIds.Cut);
		}

		private void copyButton_Click(object sender, EventArgs e)
		{
            ExecuteCommand(FormControlCommandIds.CommandIds.Copy);
		}

		private void pasteButton_Click(object sender, EventArgs e)
		{
            ExecuteCommand(FormControlCommandIds.CommandIds.Paste);
		}
		#endregion

		private void openButton_Click(object sender, EventArgs e)
		{
			bool shouldClose = SaveIfDirty();
			if (shouldClose)
			{
				OpenFileDialog openFileDialog = new OpenFileDialog();

				openFileDialog.Filter = "All InfoPath Forms (*.xsn; *.xsf; *.xml)|*.xsn;*.xsf;*.xml";
				openFileDialog.FilterIndex = 1;

				if (openFileDialog.ShowDialog() == DialogResult.OK)
				{
					string fileName = openFileDialog.FileName;
					FileInfo fileInfo = new FileInfo(fileName);
					string extension = fileInfo.Extension;

                    formControl1.Close();	// Must close existing documents before opening new ones

					try
					{
						if (extension == ".xml")
						{
							formControl1.Open(fileName);
                            _currentlySelectedForm = NoSelection;
                            comboBox1.SelectedIndex = _currentlySelectedForm;
                        }
						else if (extension == ".xsn" || extension == ".xsf")
						{
							formControl1.NewFromFormTemplate(fileName);
							_currentlySelectedForm = NoSelection;
                            comboBox1.SelectedIndex = _currentlySelectedForm;
						}
						else
						{
                            MessageBox.Show("The file you selected is not a valid InfoPath file", HostDisplayName);
						}
					}
					catch (Exception ex)
					{
                        MessageBox.Show(ex.Message, HostDisplayName);
					}
				}
			}
		}

		private void undoButton_Click(object sender, EventArgs e)
		{
            ExecuteCommand(FormControlCommandIds.CommandIds.Undo);
		}

		private void redoButton_Click(object sender, EventArgs e)
		{
            ExecuteCommand(FormControlCommandIds.CommandIds.Redo);
		}

		private void saveButton_Click(object sender, EventArgs e)
		{
			if (formControl1.XmlForm != null)
			{
				SaveForm();
			}
		}

		private bool SaveIfDirty()
		{
			if (formControl1.XmlForm != null && formControl1.XmlForm.Dirty)
			{
				DialogResult result = MessageBox.Show("Would you like to save your changes?",
                                                      HostDisplayName, 
													  MessageBoxButtons.YesNoCancel);
				if (result == DialogResult.Yes)
				{
					SaveForm();
				}
				else if (result == DialogResult.Cancel)
				{
					return false;
				}
			}

			return true;	// No form open or the form isn't dirty
		}

		private void SaveForm()
		{
			if (formControl1 != null)
			{
				if (_currentFileName != null)
				{
					formControl1.XmlForm.Save();
				}
				else
				{
					SaveFileDialog saveFileDialog = new SaveFileDialog();

					saveFileDialog.Filter = "All InfoPath Forms (*.xsn; *.xsf; *.xml)|*.xsn;*.xsf;*.xml";
					saveFileDialog.FilterIndex = 1;

					if (saveFileDialog.ShowDialog() == DialogResult.OK)
					{
						_currentFileName = saveFileDialog.FileName;
						formControl1.XmlForm.SaveAs(_currentFileName);
					}
				}
			}
		}

        private void findButton_Click(object sender, EventArgs e)
        {
            object output = null;

            // Only set the find string if it's not already set. Otherwise, we will only
            // ever find the first occurrence of the string.
            ExecuteCommand(FormControlCommandIds.CommandIds.GetFindString, null, ref output);
            if (output.ToString() != findTextBox.Text)
            {
                ExecuteCommand(FormControlCommandIds.CommandIds.SetFindString, findTextBox.Text, ref output);
            }

            ExecuteCommand(FormControlCommandIds.CommandIds.SetFindReplaceOptionSearchDirection, FindDirection.entireDocument, ref output);

            if (ExecuteCommand(FormControlCommandIds.CommandIds.FindReplaceFindNext) == false)
            {
                // Reset the find string and try again
                ExecuteCommand(FormControlCommandIds.CommandIds.SetFindString, findTextBox.Text, ref output);

                if (ExecuteCommand(FormControlCommandIds.CommandIds.FindReplaceFindNext) == false)
                {
                    MessageBox.Show(findTextBox.Text + " could not be found.", HostDisplayName);
                }
            }
        }

        private void findTextBox_TextChanged(object sender, EventArgs e)
        {
            findButton.Enabled = (findTextBox.Text != "");
        }
	}
}