using Microsoft.Office.InfoPath;
using System;
using System.Windows.Forms;
using System.Xml;
using System.Xml.XPath;

namespace MoiConsultingRequest
{
    public partial class FormCode
    {
        public void InternalStartup()
        {
            EventManager.XmlEvents["/my:Request/my:Welcome/my:RequestType"].Changing += new XmlChangingEventHandler(RequestType_Changing);
            EventManager.FormEvents.Loading += new LoadingEventHandler(FormEvents_Loading);
            ((ButtonEvent)EventManager.ControlEvents["Start_Over"]).Clicked += new ClickedEventHandler(Start_Over_Clicked);
            ((ButtonEvent)EventManager.ControlEvents["click_here_to_sign"]).Clicked += new ClickedEventHandler(click_here_to_sign_Clicked);
            EventManager.FormEvents.ContextChanged += new ContextChangedEventHandler(FormEvents_ContextChanged);
            EventManager.XmlEvents["/my:Request/my:Welcome/my:RequestType"].Changed += new XmlChangedEventHandler(RequestType_Changed);
            EventManager.FormEvents.ViewSwitched += new ViewSwitchedEventHandler(FormEvents_ViewSwitched);
            ((ButtonEvent)EventManager.ControlEvents["Add_Additional_Details"]).Clicked += new ClickedEventHandler(Add_Additional_Details_Clicked);
            EventManager.FormEvents.Submit += new SubmitEventHandler(FormEvents_Submit);
            EventManager.FormEvents.Sign += new SignEventHandler(FormEvents_Sign);
        }

        public void RequestType_Changing(object sender, XmlChangingEventArgs e)
        {
            if (e.NewValue == "SensitiveCritical" && e.Operation == XmlOperation.ValueChange)
            {
                DialogResult result = MessageBox.Show("You will be contacted within 5 minutes. Are you sure?", "High Priority Request", MessageBoxButtons.YesNo);
                if (result != DialogResult.Yes)
                    e.CancelableArgs.Cancel = true;
            }
        }

        const string _startOverOuterXml = "startOverOuterXml";
        const string _startOverGetSubAreas = "startOverGetSubAreas";
        const string _signingAllowed = "signingAllowed";

        private string StartOverOuterXml
        {
            get
            {
                if (FormState.Contains(_startOverOuterXml))
                {
                    return FormState[_startOverOuterXml] as string;
                }
                throw new Exception("Form load did not load properly before Starting Over!");
            }
        }

        private string StartOverGetSubAreas
        {
            get
            {
                if (FormState.Contains(_startOverGetSubAreas))
                {
                    return FormState[_startOverGetSubAreas] as string;
                }
                throw new Exception("Form load did not load properly before Starting Over!");
            } 
        }

        private bool SigningAllowed
        {
            get
            {
                if (FormState.Contains(_signingAllowed))
                    return (bool)FormState[_signingAllowed];
                return false;
            }
            set
            {
                if (!FormState.Contains(_signingAllowed))
                    FormState.Add(_signingAllowed, value);
                else
                    FormState[_signingAllowed] = value;
            }
        }

        public void FormEvents_Loading(object sender, LoadingEventArgs e)
        {
            // Only allow new forms (i.e. no saved forms can be opened).
            // We could add more logic to check the user role or specific data
            //  in the form before rejecting it.
            if (!this.New)
            {
                e.CancelableArgs.Cancel = true;
                e.CancelableArgs.Message = "I'm sorry, you must fill out a new request. Previous requests cannot be honored.";
                return;
            }

            XPathNavigator root = MainDataSource.CreateNavigator();
            XPathNavigator user = root.SelectSingleNode("/my:Request/my:Welcome/my:CurrentUser", NamespaceManager);
            user.SetValue(Application.User.UserName);

            XPathNavigator uri = root.SelectSingleNode("/my:Request/@my:CacheId", NamespaceManager);
            uri.SetValue(this.Template.CacheId);

            XPathNavigator version = root.SelectSingleNode("/my:Request/@my:FormVersion", NamespaceManager);
            version.SetValue(this.Template.Version);

            // Cache the Entire data source for the "Start Over" feature
            // The StartOverOuterXml property will return this saved data
            XPathNavigator docElem = root.SelectSingleNode("*");
            FormState.Add(_startOverOuterXml, docElem.OuterXml);

            // Cache secondary data source
            XPathNavigator getSubAreasRoot = DataSources["GetSubAreas"].CreateNavigator();
            XPathNavigator getSubAreasDomElem = getSubAreasRoot.SelectSingleNode("*");
            FormState.Add(_startOverGetSubAreas, getSubAreasDomElem.OuterXml);

            // Set the default view
            ViewInfos.Initial = ViewInfos["Welcome"]; 
        }

        public void Start_Over_Clicked(object sender, ClickedEventArgs e)
        {
            // Every button ID called "Start_Over" calls into this event handler
            // Go back to the main view.
            ViewInfos.SwitchView("Welcome");

            // Tell the user to remove their digital signature before we can start over
            if (this.Signed)
            {
                MessageBox.Show("Cannot start over because the form data is signed. "
                    + "Please go to the Digital Signatures dialog and remove your signature. "
                    + "Then click this button again to continue.");
                return;
            }

            // Clear the main data source
            XPathNavigator docElem = MainDataSource.CreateNavigator().SelectSingleNode("*");
            docElem.ReplaceSelf(StartOverOuterXml);

            // Clear any secondary data sources
            // Since secondary data sources don't validate, this is fine (and easy!)
            XPathNavigator subAreasRoot = DataSources["GetSubAreas"].CreateNavigator();
            subAreasRoot.SelectSingleNode("*").ReplaceSelf(StartOverGetSubAreas);
        }

        public void click_here_to_sign_Clicked(object sender, ClickedEventArgs e)
        {
            // This button click does two things in this order:
            // 1. Check only the data in this view (Welcome) for errors
            // 2. Partially sign the Welcome group

            // Pre-check: this part cannot already be signed
            if (this.Signed)
            {
                // Just go to the next view
                // Note: There can't be errors otherwise we couldn't have signed the data.
                //  As a result, it's okay to assume no errors and jump to the next view.
                ViewInfos.SwitchView("RequestDetails");
                return;
            }

            // 1. Check for errors
            XPathNavigator root = MainDataSource.CreateNavigator();
            XPathNavigator welcome = root.SelectSingleNode("/my:Request/my:Welcome", NamespaceManager);

            string welcomeErrors = string.Empty;
            foreach (FormError error in Errors)
            {
                // is this error under the "Welcome" group?
                if (welcome.IsDescendant(error.Site))
                {
                    welcomeErrors += error.Site.LocalName + ": " + error.Message + System.Environment.NewLine;
                }
            }
            
            // if there are errors, report them and stop
            if (!string.IsNullOrEmpty(welcomeErrors))
            {
                MessageBox.Show("Please fix these errors: "
                    + System.Environment.NewLine + System.Environment.NewLine
                    + welcomeErrors, "Please Fill or Fix Some Data");
                return;
            }

            // Tell the Signed event it's okay to allow signing
            SigningAllowed = true;

            // continue with signing
            foreach (SignedDataBlock sig in SignedDataBlocks)
            {
                // only sign this block
                if (sig.XPath.Expression.EndsWith("my:Welcome"))
                {
                    // Calling Sign requires a full trust form
                    try
                    {
                        sig.Sign();
                    }
                    catch (InvalidOperationException)
                    {
                        // We don't care about InvalidOperationExceptions
                    }

                    // Was it actually signed or cancelled?
                    if (sig.Signatures.Count > 0)
                    {
                        ViewInfos.SwitchView("RequestDetails");
                    }
                    else
                    {
                        MessageBox.Show("You must sign the form before continuing. Please try again.");
                    }
                    break;
                }
            }
            SigningAllowed = false;
        }

        public void FormEvents_ContextChanged(object sender, ContextChangedEventArgs e)
        {
            if (e.ChangeType == "ContextNode")
            {
                HtmlTaskPane htmlTaskPane = (HtmlTaskPane)this.TaskPanes[TaskPaneType.Html];
                if (htmlTaskPane.Visible)
                {
                    htmlTaskPane.Navigate("MoiConsultingRequest.htm#" + e.Context.LocalName);
                }
                return;
            }
        }

        public void RequestType_Changed(object sender, XmlEventArgs e)
        {
            // Only process value changes for the RequestType field.
            // If we didn't do this check, the below code would run
            //  when the user clicks a "Start Over" button because every
            //  data source field/group XML notification is fired when
            //  ReplaceSelf is called on the document element. ReplaceSelf
            //  deletes and reinserts every field/group in the data source.
            // This code also should not run when filling the OtherType field.
            if (e.UndoRedo
                || e.Operation != XmlOperation.ValueChange
                || e.Site.Name != ((XPathNavigator)sender).Name)
            {
                return;
            }

            // Request Type changed, so we need to get an updated list of Sub Areas
            DataSource getSubAreasDS = DataSources["GetSubAreas"];
            // We could have also used DataConnections["GetSubAreas"]
            WebServiceConnection getSubAreasWSC = (WebServiceConnection)getSubAreasDS.QueryConnection;
            // Get the navigator for the GetSubAreas secondary data source
            XPathNavigator getSubAreasNav = getSubAreasDS.CreateNavigator();
            // Get the requestType query parameter that will be sent to the Web service
            XPathNavigator requestTypeNav = getSubAreasNav.SelectSingleNode("/dfs:myFields/dfs:queryFields/tns:GetSubAreas/tns:requestType", NamespaceManager);
            // Get the Request Type from the main data source (the node for this handler!)
            requestTypeNav.SetValue(e.NewValue);

            // Query the Web service with the new parameter value
            // Web service timeout is 30s, increase it to 60 for slow connections
            getSubAreasWSC.Timeout = 60;
            // Create a new navigator to capture errors, if any
            XmlDocument errorsXmlDoc = new XmlDocument();
            XPathNavigator errorsNav = errorsXmlDoc.CreateNavigator();
            try
            {
                // Query the Web service
                getSubAreasWSC.Execute(null /*input*/, null /*output*/, errorsNav /*errors*/);
            }
            catch (Exception)
            {
                // Silently fail for now and we'll show the message box in a moment.
                // If we didn't do this, InfoPath would show the exception to the user.
            }

            // Did an error occur?
            if (errorsNav.HasChildren)
            {
                MessageBox.Show("I'm sorry, and error occurred accessing " + getSubAreasWSC.ServiceUrl + ". Please select a Request Type again.");
            }
        }

        public void FormEvents_ViewSwitched(object sender, ViewSwitchedEventArgs e)
        {
            switch(this.CurrentView.ViewInfo.Name)
            {
                case "Confirm":
                    MessageBox.Show("Please confirm the data and click Submit to continue.", "Confirm Data");
                    break;
                case "Welcome":
                    Application.ActiveWindow.TaskPanes[TaskPaneType.Html].Visible = true;
                    break;
                default:
                    Application.ActiveWindow.TaskPanes[TaskPaneType.Html].Visible = false;
                    break;
            }
        }

        public void Add_Additional_Details_Clicked(object sender, ClickedEventArgs e)
        {
            // Is AdditionalDetails already inserted?
            XPathNavigator root = MainDataSource.CreateNavigator();
            XPathNavigator additionalDetails = root.SelectSingleNode("/my:Request/my:RequestDetails/my:AdditionalDetails", NamespaceManager);

            // If it doesn't exist, we'll insert it
            if (additionalDetails == null)
            {
                // Insert the AdditionalDetails optional section
                // XmlToEdit for xOptional: AdditionalDetailsSection_2
                this.CurrentView.ExecuteAction(ActionType.XOptionalInsert, "AdditionalDetailsSection_2");
            }
            // If it already exists, insert an AdditionalDetail (child of AdditionalDetails)
            //  which is bound to a Bulleted List
            else
            {
                // Bulleted List doesn't support ExecuteAction since there's no XmlToEdit.
                // We'll insert it using the data source instead.
                // Args: prefix, name, namespaceUri, value
                additionalDetails.AppendChildElement(
                    additionalDetails.Prefix, "AdditionalDetail",
                    additionalDetails.NamespaceURI, string.Empty);
            }
        }

        public void FormEvents_Submit(object sender, SubmitEventArgs e)
        {
            // NOTE: there are no errors if InfoPath allows submit to happen

            // If this is recovered data, ask the user if we should continue.
            DialogResult result = DialogResult.Yes;
            if (this.Recovered)
            {
                result = MessageBox.Show("This data is recovered. Still continue?", "Submitting Recovered Data", MessageBoxButtons.YesNo);
            }

            if (result == DialogResult.Yes)
            {
                // Save the form
                string tempFileName = System.IO.Path.GetTempFileName() + ".xml";
                this.SaveAs(tempFileName);
                // Fills a link to see where it saved
                XPathNavigator root = MainDataSource.CreateNavigator();
                XPathNavigator savedFileLocation = root.SelectSingleNode("/my:Request/@my:SavedFileLocation", NamespaceManager);
                savedFileLocation.SetValue(tempFileName);
                // Go to the ThankYou view
                ViewInfos.SwitchView("ThankYou");
            }
            else
            {
                e.CancelableArgs.Cancel = true;
                e.CancelableArgs.Message = "Submit was cancelled.";
                e.CancelableArgs.MessageDetails = "Submit again whenever you are ready.";
            }
        }

        public void FormEvents_Sign(object sender, SignEventArgs e)
        {
            // The Sign handler can be customized only in fully trusted form templates.

            if (SigningAllowed)
            {
                Signature thisSignature = e.SignedDataBlock.Signatures.CreateSignature();
                thisSignature.Sign();
                e.SignatureWizard = true;
            }
            else
            {
                e.SignatureWizard = false;
                MessageBox.Show("Please use the 'click here' button (in the form) to proceed.");
                return;
            }
        }
    }
}

