using Microsoft.Office.InfoPath;
using System;
using System.Drawing;
using System.Windows.Forms;
using System.Xml;
using System.Xml.XPath;

namespace MoiEventScheduler
{
    public partial class FormCode
    {
        private const string UserIsAuthorizedStr = "UserIsAuthorized";

        public void InternalStartup()
        {
            EventManager.FormEvents.Loading += new LoadingEventHandler(FormEvents_Loading);
            EventManager.XmlEvents["/my:myFields"].Changing += new XmlChangingEventHandler(myFields_Changing);
            EventManager.XmlEvents["/my:myFields/my:StartDateTime"].Validating += new XmlValidatingEventHandler(StartDateTime_Validating);
            EventManager.XmlEvents["/my:myFields/my:EndDateTime"].Validating += new XmlValidatingEventHandler(EndDateTime_Validating);
            EventManager.XmlEvents["/my:myFields/my:WaiverNeeded"].Validating += new XmlValidatingEventHandler(WaiverNeeded_Validating);
        }

        /// <summary>
        /// Remembers if this user is authorized.
        /// </summary>
        private bool UserIsAuthorized
        {
            get
            {
                return (bool)FormState[UserIsAuthorizedStr];
            }
            set
            {
                FormState[UserIsAuthorizedStr] = value;
            }
        }

        public void myFields_Changing(object sender, XmlChangingEventArgs e)
        {
            // Once a user is validated we won't prompt again
            if (UserIsAuthorized)
                return;

            string password = new MyPasswordDialog().Prompt();

            // Note: Including passwords as clear text in code is poor security
            if (password == "InfoPath!")
            {
                // Allow access
                e.CancelableArgs.Cancel = false;
                // Remember they have access
                UserIsAuthorized = true;
                MessageBox.Show("Thank you. Access has been granted for this session.", "Access Granted");
            }
            else
            {
                // Deny access with a dialog
                e.CancelableArgs.Cancel = true;
                e.CancelableArgs.Message = "Authorized Users Only";
                e.CancelableArgs.MessageDetails = "Only authorized users can schedule morale events.";
            }
        }

        public void FormEvents_Loading(object sender, LoadingEventArgs e)
        {
            FormState.Add(UserIsAuthorizedStr, false);
        }

        public void StartDateTime_Validating(object sender, XmlValidatingEventArgs e)
        {
            // Get Start and End date-times
            XPathNavigator root = MainDataSource.CreateNavigator();
            XPathNavigator start = e.Site;
            XPathNavigator end = root.SelectSingleNode("/my:myFields/my:EndDateTime", NamespaceManager);
            DateTime startDT, endDT;

            // Can we read the start?
            if (!DateTime.TryParse(start.Value, out startDT))
                return; // InfoPath handles invalid DateTime values

            const string error2a = "Start time is between 8 AM and 4 PM";
            if (startDT.Hour < 8 /*8am*/ || startDT.Hour > 12 + 4 /*4pm*/)
                e.ReportError(start, false /*siteIndepedent*/, error2a);

            // Can we read the end?
            if (!DateTime.TryParse(end.Value, out endDT))
                return; // InfoPath handles invalid DateTime values

            CheckDateTimes(e, startDT, endDT);
        }

        public void EndDateTime_Validating(object sender, XmlValidatingEventArgs e)
        {
            // Get Start and End date-times
            XPathNavigator root = MainDataSource.CreateNavigator();
            XPathNavigator start = root.SelectSingleNode("/my:myFields/my:StartDateTime", NamespaceManager);
            XPathNavigator end = e.Site;
            DateTime startDT, endDT;

            // Can we read the end?
            if (!DateTime.TryParse(end.Value, out endDT))
                return; // InfoPath handles invalid DateTime values

            const string error2b = "End time is between 9 AM and 10 PM";
            if (endDT.Hour < 9 /*9am*/ || endDT.Hour > 12 + 10 /*10pm*/)
                e.ReportError(end, false /*siteIndepedent*/, error2b + "-" + e.Site.Name);

            // Can we read the start?
            if (!DateTime.TryParse(start.Value, out startDT))
                return; // InfoPath handles invalid DateTime values

            CheckDateTimes(e, startDT, endDT);
        }

        private void CheckDateTimes(XmlValidatingEventArgs e, DateTime startDT, DateTime endDT)
        {
            const string error1 = "Requested Start date/time is later than the End date/time.";
            const string error3 = "An event may not last longer than 1 day if a waiver is necessary.";
            if (startDT > endDT)
                e.ReportError(e.Site, false /*siteIndepedent*/, error1);

            if (((TimeSpan)endDT.Subtract(startDT)).Days >= 1 && WaiverNeeded)
                e.ReportError(e.Site, false /*siteIndependent*/, error3);
        }

        private bool WaiverNeeded
        {
            get
            {
                XPathNavigator root = MainDataSource.CreateNavigator();
                XPathNavigator waiver = root.SelectSingleNode("/my:myFields/my:WaiverNeeded", NamespaceManager);
                return waiver.ValueAsBoolean;
            }
        }

        public void WaiverNeeded_Validating(object sender, XmlValidatingEventArgs e)
        {
            // Get Start and End date-times
            XPathNavigator root = MainDataSource.CreateNavigator();
            XPathNavigator start = root.SelectSingleNode("/my:myFields/my:StartDateTime", NamespaceManager);
            XPathNavigator end = root.SelectSingleNode("/my:myFields/my:EndDateTime", NamespaceManager);
            DateTime startDT, endDT;

            // Can we parse the Start and End?
            if (!DateTime.TryParse(end.Value, out endDT) || !DateTime.TryParse(start.Value, out startDT))
            {
                return; // InfoPath handles invalid DateTime values
            }

            // Only do start and end common checks if date-times are both valid
            if (DateTimesAreValid())
                CheckDateTimes(e, startDT, endDT);
        }

        /// <summary>
        /// True if both Start and End Date/Times are valid
        /// </summary>
        private bool DateTimesAreValid()
        {
            // Are the Start and End date-times valid?
            foreach (FormError err in Errors.GetErrors(FormErrorType.SystemGenerated))
            {
                if (err.Site.Name == "my:StartDateTime" || err.Site.Name == "my:EndDateTime")
                {
                    return false;
                }
            }
            return true;
        }
    }

    /// <summary>
    /// My own dialog for prompting a user for a password.
    /// </summary>
    public class MyPasswordDialog : Form
    {
        TextBox pwdTextbox;
        Button okButton, cancelButton;
        FlowLayoutPanel flow;
        DialogResult result;

        public MyPasswordDialog()
        {
            // Password text box
            pwdTextbox = new TextBox();
            pwdTextbox.MaxLength = 15;
            pwdTextbox.UseSystemPasswordChar = true;
            pwdTextbox.Size = new Size(150, pwdTextbox.Size.Width);
            // Catch Enter and Esc keys for convenience
            pwdTextbox.KeyDown += new KeyEventHandler(pwdTextbox_KeyDown);

            // OK button
            okButton = new Button();
            okButton.Text = "OK";
            okButton.Click += new EventHandler(okButton_Click);

            // Cancel button
            cancelButton = new Button();
            cancelButton.Text = "Cancel";
            cancelButton.Click += new EventHandler(cancelButton_Click);

            // Flow layout
            flow = new FlowLayoutPanel();
            flow.Dock = DockStyle.Fill;
            flow.Location = new Point(4, 4);
            flow.Size = new Size(100, 300);
            flow.TabIndex = 0;

            // This form
            Text = "Enter a password to continue";
            AutoSize = true;
            ClientSize = new Size(185, 65);
            Controls.Add(flow);
            FormBorderStyle = FormBorderStyle.FixedToolWindow;
            MaximizeBox = false;
            MinimizeBox = false;

            // Default for result
            result = DialogResult.Cancel;
        }

        // Handles keys pressed in the password dialog
        void pwdTextbox_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.KeyCode == Keys.Escape)
                cancelButton_Click(null, null);
            else if (e.KeyCode == Keys.Enter)
                okButton_Click(null, null);
        }

        // Handles Cancel button click
        void cancelButton_Click(object sender, EventArgs e)
        {
            result = DialogResult.Cancel;
            Close();
        }

        // Handles OK button click
        void okButton_Click(object sender, EventArgs e)
        {
            result = DialogResult.OK;
            Close();
        }

        // Show prompt and return password
        public string Prompt()
        {
            flow.Controls.Add(pwdTextbox);
            flow.Controls.Add(okButton);
            flow.Controls.Add(cancelButton);
            flow.ResumeLayout(false);
            flow.PerformLayout();
            PerformLayout();
            ShowDialog();

            if (result == DialogResult.OK)
                return pwdTextbox.Text;
            return string.Empty;
        }
    }
}
            
