using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Text;
using System.Xml;
using System.Windows.Forms;
using System.Runtime.InteropServices;
using Microsoft.Office.Interop.InfoPath;
using Microsoft.Office.Interop.InfoPath.Xml;

namespace InfoPathControlLibrary
{
    #region IInfoPathMaskedTextBox
    [InterfaceType(ComInterfaceType.InterfaceIsIDispatch)]
    public interface IInfoPathMaskedTextBox
    {
        [DispId(InfoPathMaskedTextBox.DispidValue)]
        string Value { get; set; }

        [DispId(InfoPathMaskedTextBox.DispidEnabled)]
        bool Enabled { get; set; }
    }
    #endregion

    #region IObjectSafety
    [ComImport]
    [Guid("CB5BDC81-93C1-11CF-8F20-00805F2CD064")]
    [InterfaceType(ComInterfaceType.InterfaceIsIUnknown)]
    interface IObjectSafety
    {
        [PreserveSig]
        int GetInterfaceSafetyOptions(ref Guid riid, out int pdwSupportedOptions, out int pdwEnabledOptions);

        [PreserveSig]
        int SetInterfaceSafetyOptions(ref Guid riid, int dwOptionSetMask, int dwEnabledOptions);
    }
    #endregion

    #region IPropertyNotifySink
    [ComImport]
    [Guid("9BFBBC02-EFF1-101A-84ED-00AA00341D07")]
    [InterfaceType(ComInterfaceType.InterfaceIsIUnknown)]
    public interface IPropertyNotifySink
    {
        [PreserveSig]
        int OnChanged(int dispId);

        [PreserveSig]
        int OnRequestEdit(int dispId);
    }

    public delegate int OnChangedDelegate(int dispID);
    #endregion

    [ComVisible(true)]
    [Guid("F24E3AC4-3987-4a62-9DDC-AB56CE896301")]
    [ClassInterface(ClassInterfaceType.AutoDual)]   // AutoDual is required for the control to work in InfoPath
    [ComSourceInterfaces(typeof(IPropertyNotifySink))]
    public partial class InfoPathMaskedTextBox : UserControl, IInfoPathMaskedTextBox, IObjectSafety, InfoPathControl
    {
        #region Data Members
        private InfoPathControlSite _controlSite;
        private bool _fDirty;
        #endregion

        #region DispatchIDs
        // DispidValue must be 0 if the binding property is Value
        // since COM interop will return 0 for DISPID_VALUE when 
        // queried by InfoPath. Using Value as the binding property
        // is the easiest way to hook up binding in InfoPath. In
        // that case, the Dispatch ID will always be 0. 
        internal const int DispidValue = 0;
        internal const int DispidEnabled = 1;
        #endregion

        public InfoPathMaskedTextBox()
        {
            InitializeComponent();
        }

        #region IInfoPathMaskedTextBox properties
        public string Value
        {
            get
            {
                if (_controlSite != null)
                {
                    // Uncomment the next line to see the XML of the bound node.
                    //MessageBox.Show("Value (get) called: " + _controlSite.Node.xml);
                }

                return maskedTextBox1.Text;
            }

            set
            {
                if (_controlSite != null)
                {
                    // Uncomment the next line to see the XML of the bound node.
                    //MessageBox.Show("Value (set) called: " + _controlSite.Node.xml);
                }

                maskedTextBox1.Text = value;
            }
        }

        public new bool Enabled
        {
            get
            {
                return maskedTextBox1.Enabled;
            }

            set
            {
                maskedTextBox1.Enabled = value;
            }
        }
        #endregion

        #region IObjectSafety Implementation
            #region IObjectSafety Constants (from objsafe.h in the Platform SDK)
            private const int INTERFACESAFE_FOR_UNTRUSTED_CALLER = 1;
            private const int INTERFACESAFE_FOR_UNTRUSTED_DATA = 2;
            #endregion

            #region COM Constants
            private const int S_OK = 0;
            #endregion

        int IObjectSafety.GetInterfaceSafetyOptions(ref Guid riid, out int pdwSupportedOptions, out int pdwEnabledOptions)
        {
            pdwSupportedOptions = INTERFACESAFE_FOR_UNTRUSTED_CALLER | INTERFACESAFE_FOR_UNTRUSTED_DATA;
            pdwEnabledOptions = INTERFACESAFE_FOR_UNTRUSTED_CALLER | INTERFACESAFE_FOR_UNTRUSTED_DATA;

            return S_OK;
        }

        int IObjectSafety.SetInterfaceSafetyOptions(ref Guid riid, int dwOptionSetMask, int dwEnabledOptions)
        {
            return S_OK;
        }
        #endregion

        #region IPropertyNotifySink helper methods
        public event OnChangedDelegate OnChanged;

        private int FireOnChanged(int dispID)
        {
            if (OnChanged != null)
            {
                return OnChanged(dispID);
            }

            return 0;
        }
        #endregion

        #region InfoPathControl Implementation
        public void Init(InfoPathControlSite controlSite)
        {
            _controlSite = controlSite;
        }

        public void Uninit()
        {
            _controlSite = null;
        }

        public void SaveState()
        {
        }

        public void Enable(bool enabled)
        {
            this.Enabled = enabled;
        }
        #endregion

        #region Private Methods
        private void maskedTextBox1_TextChanged(object sender, System.EventArgs e)
        {
            _fDirty = true;
        }

        private void maskedTextBox1_LostFocus(object sender, System.EventArgs e)
        {
            // Note: if your binding property is not named Value, 
            // pass DISPID_UNKNOWN (-1) to FireOnChanged.
            if (_fDirty)
            {
                FireOnChanged(DispidValue);
                _fDirty = false;
            }
        }
        #endregion
    }
}
