namespace FormUtilities
{
    using System;
    using Extensibility;
    using System.Runtime.InteropServices;
    using Microsoft.Office.Core;
    using Microsoft.Office.Interop.InfoPath;
    using Microsoft.Office.Interop.InfoPath.Xml;
    using System.Windows.Forms;
    using System.Collections;
    using System.Reflection;
    using System.Diagnostics;

    #region Read me for Add-in installation and setup information.
    // When run, the Add-in wizard prepared the registry for the Add-in.
    // At a later time, if the Add-in becomes unavailable for reasons such as:
    //   1) You moved this project to a computer other than which is was originally created on.
    //   2) You chose 'Yes' when presented with a message asking if you wish to remove the Add-in.
    //   3) Registry corruption.
    // you will need to re-register the Add-in by building the FormUtilitiesSetup project, 
    // right click the project in the Solution Explorer, then choose install.
    #endregion

    /// <summary>
    ///   The object for implementing an Add-in.
    /// </summary>
    /// <seealso class='IDTExtensibility2' />
    [GuidAttribute("29D8627D-71C3-4D0A-8249-DB1434F039FF"), ProgId("FormUtilities.Connect")]
    public class Connect : Object, Extensibility.IDTExtensibility2
    {
        /// <summary>
        ///		Implements the constructor for the Add-in object.
        ///		Place your initialization code within this method.
        /// </summary>
        public Connect()
        {
        }

        /// <summary>
        ///      Implements the OnConnection method of the IDTExtensibility2 interface.
        ///      Receives notification that the Add-in is being loaded.
        /// </summary>
        /// <param term='application'>
        ///      Root object of the host application.
        /// </param>
        /// <param term='connectMode'>
        ///      Describes how the Add-in is being loaded.
        /// </param>
        /// <param term='addInInst'>
        ///      Object representing this Add-in.
        /// </param>
        /// <seealso class='IDTExtensibility2' />
        public void OnConnection(
                        object application,
                        Extensibility.ext_ConnectMode connectMode,
                        object addInInst,
                        ref System.Array custom)
        {
            try
            {
                _applicationObject = (ApplicationClass)application;
            }
            catch (Exception)
            {
                MessageBox.Show("The Form Utilities Add-in is only supported in Microsoft Office InfoPath", DialogCaption);
                throw;
            }

            if (connectMode == ext_ConnectMode.ext_cm_Startup)
            {
                // _applicationEvents has to be a data member in order
                // for the event sinks to remain attached throughout the 
                // lifetime of the application.
                _applicationEvents = (ApplicationEvents)_applicationObject.Events;

                _applicationEvents.XDocumentOpen += new _ApplicationEvents_XDocumentOpenEventHandler(_applicationEvents_XDocumentOpen);
                _applicationEvents.NewXDocument += new _ApplicationEvents_NewXDocumentEventHandler(_applicationEvents_NewXDocument);
                _applicationEvents.XDocumentBeforeClose += new _ApplicationEvents_XDocumentBeforeCloseEventHandler(_applicationEvents_XDocumentBeforeClose);
            }
        }

        /// <summary>
        /// Called when an existing form is opened
        /// </summary>
        private void _applicationEvents_XDocumentOpen(_XDocument pDocument)
        {
            ((XDocument)pDocument).OnSwitchView += new _XDocumentEventSink2_OnSwitchViewEventHandler(Connect_OnSwitchView);
        }
  
        /// <summary>
        /// Called when a new form is created from a form template
        /// </summary>
        void _applicationEvents_NewXDocument(_XDocument pDocument)
        {
            ((XDocument)pDocument).OnSwitchView += new _XDocumentEventSink2_OnSwitchViewEventHandler(Connect_OnSwitchView);
        }

        /// <summary>
        /// Called when switching to a new view or when the 
        /// form is initially loaded.
        /// Note that OnSwitchView may not fire if there is
        /// an event handler in business logic for the form.
        /// </summary>
        void Connect_OnSwitchView(DocEvent pEvent)
        {
            CommandBarButton copyButton = GetButton(CopyButtonTag);
            copyButton.Enabled = true;

            CommandBarButton pasteButton = GetButton(PasteButtonTag);
            pasteButton.Enabled = (_copiedNode != null);
        }

        /// <summary>
        /// Called when the document is about to close.
        /// </summary>
        void _applicationEvents_XDocumentBeforeClose(_XDocument pDocument, ref bool pfCancel)
        {
            // When the document is closed due to the user closing the application, the
            // ActiveWindow object is released by the time this event handler is called.
            // This also means that the toolbars have been released. 
            WindowObject window = _applicationObject.ActiveWindow;
            if (window != null)
            {
                // Disable the buttons for this window only.
                CommandBarButton copyButton = GetButton(CopyButtonTag);
                CommandBarButton pasteButton = GetButton(PasteButtonTag);

                copyButton.Enabled = false;
                pasteButton.Enabled = false;
            }
        }

        /// <summary>
        ///     Implements the OnDisconnection method of the IDTExtensibility2 interface.
        ///     Receives notification that the Add-in is being unloaded.
        /// </summary>
        /// <param term='disconnectMode'>
        ///      Describes how the Add-in is being unloaded.
        /// </param>
        /// <param term='custom'>
        ///      Array of parameters that are host application specific.
        /// </param>
        /// <seealso class='IDTExtensibility2' />
        public void OnDisconnection(Extensibility.ext_DisconnectMode disconnectMode, ref System.Array custom)
        {
            _applicationObject = null;
            _applicationEvents = null;
            _copiedNode = null;
        }

        /// <summary>
        ///      Implements the OnAddInsUpdate method of the IDTExtensibility2 interface.
        ///      Receives notification that the collection of Add-ins has changed.
        /// </summary>
        /// <param term='custom'>
        ///      Array of parameters that are host application specific.
        /// </param>
        /// <seealso class='IDTExtensibility2' />
        public void OnAddInsUpdate(ref System.Array custom)
        {
        }

        /// <summary>
        ///      Implements the OnStartupComplete method of the IDTExtensibility2 interface.
        ///      Receives notification that the host application has completed loading.
        /// </summary>
        /// <param term='custom'>
        ///      Array of parameters that are host application specific.
        /// </param>
        /// <seealso class='IDTExtensibility2' />
        public void OnStartupComplete(ref System.Array custom)
        {
            // The application window is now ready. We can 
            // access the CommandBars collection of the window
            // and add our button.
            CreateToolbar();

            // Now, add our button if it's not already there.
            try
            {
                // See if the copy and paste buttons already exist.
                CommandBarButton copyButton = GetButton(CopyButtonTag);
                CommandBarButton pasteButton = GetButton(PasteButtonTag);

                // Add the buttons if needed. Add the paste button first
                // so it will be after the copy button on the toolbar.
                AddButton(ref pasteButton,
                          "Paste Form Data",
                          "Paste data copied from another form",
                          PasteButtonTag,
                          true /*beginGroup*/,
                          OnPasteClick);

                AddButton(ref copyButton,
                          "Copy Form Data",
                          "Copy all data from this form",
                          CopyButtonTag,
                          false /*beginGroup*/,
                          OnCopyClick);

                copyButton.Enabled = false;
                pasteButton.Enabled = false;
            }
            catch (Exception e)
            {
                MessageBox.Show(e.Message, DialogCaption);
            }
        }

        /// <summary>
        /// Creates the toolbar for our COM Add-in 
        /// if it doesn't already exist.
        /// </summary>
        void CreateToolbar()
        {
            CommandBar toolbar = GetCurrentToolbar();
            if (toolbar == null)
            {
                Window window = _applicationObject.Windows[0];
                CommandBars commandBars = (CommandBars)window.CommandBars;

                // Note that ToolbarName is a constant defined
                // as Form Utilities which is the name of the toolbar
                // for the COM add-in.
                toolbar = commandBars.Add(
                                ToolbarName,
                                MsoBarPosition.msoBarTop,
                                false /*replace*/,
                                false /*temporary*/);
            }

            toolbar.Visible = true;
        }

        /// <summary>
        /// Returns the toolbar for the current window.
        /// </summary>
        CommandBar GetCurrentToolbar()
        {
            Window window = _applicationObject.ActiveWindow;
            if (window == null)
            {
                window = _applicationObject.Windows[0];
            }

            CommandBars commandBars = (CommandBars)window.CommandBars;
            CommandBar toolbar = null;

            // Referencing our command bar like this will throw an exception
            // if it doesn't exist. Alternatively, we could use the 
            // enumerator on the CommandBars object to look for our
            // command bar.
            try
            {
                toolbar = commandBars[ToolbarName];
            }
            catch (ArgumentException)
            {
                // An exception is thrown if the toolbar doesnt exist
            }

            return toolbar;
        }

        /// <summary>
        ///     Retrieves a button from the commandbar if the button with 
        ///     the given tag ID exists.
        /// </summary>
        /// <param term='tag'>
        ///      Tag of the button to find.
        /// </param>
        CommandBarButton GetButton(string tag)
        {
            return (CommandBarButton)GetCurrentToolbar().FindControl(
                        MsoControlType.msoControlButton,
                        Missing.Value,
                        tag,
                        true /*Visible*/,
                        false /*Recursive*/);
        }

        /// <summary>
        ///     Adds a button to the command bar.
        /// </summary>
        /// <param term='tag'>
        ///      Tag of the button to find.
        /// </param>
        void AddButton(ref CommandBarButton button,
                       string caption,
                       string tooltip,
                       string tag,
                       bool beginGroup,
                       _CommandBarButtonEvents_ClickEventHandler clickHandler)
        {
            if (button == null)
            {
                button = (CommandBarButton)GetCurrentToolbar().Controls.Add(
                                MsoControlType.msoControlButton,
                                Missing.Value,
                                Missing.Value,
                                1,
                                false /*temporary*/);

                button.BeginGroup = beginGroup;
            }

            // Reset the button properties in case something failed
            // the last time we attempted to do so.
            button.Visible = true;
            button.Caption = caption;
            button.Style = MsoButtonStyle.msoButtonCaption;
            button.Tag = tag;
            button.TooltipText = tooltip;
            button.Click += new _CommandBarButtonEvents_ClickEventHandler(clickHandler);
        }

        /// <summary>
        /// Called with the copy button is clicked.
        /// </summary>
        public void OnCopyClick(CommandBarButton Ctrl, ref bool CancelDefault)
        {
            try
            {
                XDocument document = _applicationObject.ActiveWindow.XDocument;

                IXMLDOMNode rootNode = document.DOM.selectSingleNode("*[not(self::processing-instruction())]");
                _copiedNode = rootNode.cloneNode(true /*deep*/);

                EnableAllPasteButtons();
            }
            catch (Exception e)
            {
                MessageBox.Show(e.Message);
            }
        }
        /// <summary>
        /// Enables the paste button in all opened InfoPath windows.
        /// </summary>
        private void EnableAllPasteButtons()
        {
            Windows windows = _applicationObject.Windows;
            int numWindows = windows.Count;

            for (int i = 0; i < numWindows; i++)
            {
                Window window = windows[i];
                CommandBar toolbar = ((CommandBars)window.CommandBars)[ToolbarName];

                CommandBarButton pasteButton = (CommandBarButton)toolbar.FindControl(
                                                    MsoControlType.msoControlButton,
                                                    Missing.Value,
                                                    PasteButtonTag,
                                                    true,
                                                    false);
                pasteButton.Enabled = true;
            }
        }

        /// <summary>
        /// Called with the paste button is clicked.
        /// </summary>
        public void OnPasteClick(CommandBarButton Ctrl, ref bool CancelDefault)
        {
            XDocument document = _applicationObject.ActiveWindow.XDocument;

            try
            {
                // Get the root node of the target document
                IXMLDOMNode targetRootNode = document.DOM.selectSingleNode("*[not(self::processing-instruction())]");

                // Ensure that the namespace URI of the source document 
                // matches that of the target.
                if (targetRootNode.namespaceURI == _copiedNode.namespaceURI)
                {
                    document.View.DisableAutoUpdate();

                    // Note that replaceChild sometimes fails if you have multiple InfoPath windows opened that have each
                    // loaded the same type of form. In some cases, InfoPath will throw an exception whose Message 
                    // parameter is "DOM calls that affect multiple open forms are not allowed". Other times, replaceChild
                    // will just fail silently. Using replaceChild, is a quick way to demonstrate the desired functionality.
                    // However, since it is not reliable, it would be better to walk the DOM and replace the text nodes
                    // one-by-one.
                    targetRootNode.ownerDocument.replaceChild(_copiedNode, targetRootNode);
                    document.View.EnableAutoUpdate();
                }
                else
                {
                    MessageBox.Show("You cannot paste into this document. The " +
                        "namespace of the target document does not match " +
                        "that of the source document.");
                }
            }
            catch (Exception e)
            {
                MessageBox.Show(e.Message);
            }
        }

        /// <summary>
        ///      Implements the OnBeginShutdown method of the IDTExtensibility2 interface.
        ///      Receives notification that the host application is being unloaded.
        /// </summary>
        /// <param term='custom'>
        ///      Array of parameters that are host application specific.
        /// </param>
        /// <seealso class='IDTExtensibility2' />
        public void OnBeginShutdown(ref System.Array custom)
        {
        }

        // InfoPath-specific objects
        private ApplicationClass _applicationObject;
        private ApplicationEvents _applicationEvents;
        private IXMLDOMNode _copiedNode = null;

        private const string ToolbarName = "Form Utilities";
        private const string CopyButtonTag = "Copy";
        private const string PasteButtonTag = "Paste";
        private const string DialogCaption = "FormUtilities";
    }
}